WaitTokenGuard

Struct WaitTokenGuard 

pub struct WaitTokenGuard { /* private fields */ }

Implementations

Source

impl WaitTokenGuard

pub fn new() -> Self

pub fn clone_tk(&self) -> WaitToken

Methods from Deref<Target = WaitToken>

pub fn spawn_terminator(&self) -> JoinHandle<Result<(), Error>>

Spawns background fetch to listem SIGTERM & SIGINT events and calls cancel

pub fn cancel(&self) -> CancelState

Set state to ‘cancelled’. Futures waiting for token will complete and streams terminated.

pub fn reset(&self)

Set state to ‘not cancelled’

pub fn is_cancelled(&self) -> bool

pub fn cancelled(&self) -> WaitTokenCancelled

pub fn until_cancel(&self) -> TakeUntil<Repeat<()>, WaitTokenCancelled>

Make a stream that will emit items until token is cancelled

pub fn on_cancel(&self) -> Once<WaitTokenCancelled>

pub fn repeat_until_cancel(&self, frq: Duration) -> WaitTokenRepeat<Self>

Make an object that will wait for delay every iteration and return false if token is cancelled. Common usage is with while loop.

Example:

let mut repeater = cancellation.repeat_until_cancel(Duration::from_secs(1));
while repeater.next().await {
    do_something();
}
pub async fn run_fn<Fut: Future>(&self, fut: Fut) -> Option<Fut::Output>

Run future. Future will be aborted when token is cancelled

pub fn make_child_token(&self) -> Self

Create a sub-token that will cancel if current token cancelled.
Cancel on child token will not cancel parent

pub fn guard(&self) -> WaitTokenGuard

Create a guard that will cancel wait token on drop